:- auto_table.

nonDeterministicGoal(InterestingVarsTerm,G,ListTM) :-
  findall(InterestingVarsTerm,G,L), buildTermModel(L,ListTM).

/* definition of rules */
supportive_rule(Name, Head, Body) :- strict(Name, Head, Body).
supportive_rule(Name, Head, Body) :- defeasible(Name, Head, Body).

/* definition of definitely */
definitely(X):- fact(X).
definitely(X<Y):- compute([X,+,0],K), compute([Y,+,0],M), K @< M.
definitely(X>Y):- compute([X,+,0],K), compute([Y,+,0],M), K @> M.
definitely(X>=Y):- compute([X,+,0],K), compute([Y,+,0],M), K @>= M.
definitely(X=<Y):- compute([X,+,0],K), compute([Y,+,0],M), K @=< M.
definitely(X\==Y):- X \== Y.
definitely(naf(X,Z,M)):- not(naf(X,Z,M)). 
definitely(X):- strict(R,X,L), definitely_provable(L).

/* definition of compute */
compute([H1, H2, H3|_], K2) :- compute(H1, K), compute([K, H2, H3], K2).
compute([X, Y, Z|_], K) :- number(X), number(Z), Y == *, K is X * Z.
compute([X, Y, Z|_], K) :- number(X), number(Z), Y == /, K is X / Z.
compute([X, Y, Z|_], K) :- number(X), number(Z), Y == +, K is X + Z.
compute([X, Y, Z|_], K) :- number(X), number(Z), Y == -, K is X - Z.
compute([H1, H2, H3|_], K2) :- compute(H3, K), compute([H1, H2, K], K2).

definitely_provable([X1|X2]):- definitely_provable(X1), definitely_provable(X2).
definitely_provable(X):- definitely(X).
definitely_provable([]).

/* definition of ambiguity blocking defeasibly */
defeasibly(X):- definitely(X).
defeasibly(X):- negation(X,X1), supportive_rule(R,X,L), defeasibly_provable(L), unblocked(R,X), xsb_meta_not(definitely(X1)).

defeasibly_provable([X1|X2]):- defeasibly_provable(X1), defeasibly_provable(X2).
defeasibly_provable(X):- defeasibly(X).
defeasibly_provable([]).

unblocked(R,X):- negation(X,X1), xsb_meta_not(supportive_rule(S,X1,L)).
unblocked(R,X):- negation(X,X1), xsb_meta_not(undefeated(X1)).

blocked(R,X):- supportive_rule(R,X,L), xsb_meta_not(defeasibly_provable(L)).
blocked(R,X):- supportive_rule(R,X,L), defeasibly_provable(L), defeated(R,X).

undefeated(X):- supportive_rule(S,X,L), xsb_meta_not(blocked(S,X)).
defeated(S,X):- negation(X,X1), supportive_rule(T,X1,V), defeasibly_provable(V), sup(T,S).

negation(~(X),X):- !.
negation(X,~(X)).

xsb_meta_not(X) :- not(X).

strict(t1,t2,t3).
defeasible(s1,s2,s3).
sup(t1,s1).
fact(ts).